# Generated by Django 2.1.11 on 2019-10-31 08:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0020_auto_20191030_0916'),
        ('djautotask', '0018_auto_20191029_1621'),
    ]

    operations = [
    ]
